@@ -1,13 +1,15 @@
 package File::Open;
 
-use warnings; use warnings FATAL => qw(layer);
 use strict;
+use warnings;
+BEGIN { warnings->import(FATAL => 'layer') if $] >= 5.008; }
 
-our $VERSION = '0.021';
+*VERSION = \'0.03';
 
 use File::Basename qw(basename);
 use Carp qw(croak);
 use Fcntl ();
+use Errno ();
 use Exporter qw(import);
 
 our @EXPORT;
@@ -34,6 +36,11 @@ sub _open {
 	my $b = $mode =~ s/(?<=.)b//;
 	my $emode = $modemap{$mode} or croak "Unknown $func() mode '$mode'";
 
+	if ($file =~ /\0/) {
+		$! = Errno::ENOENT() if exists &Errno::ENOENT;
+		return undef;
+	}
+
 	open my $fh, $emode, $file or return undef;
 	binmode $fh if $b;
 	binmode $fh, $layers if defined $layers;
@@ -92,6 +99,11 @@ sub _sysopen {
 		;
 	}
 
+	if ($file =~ /\0/) {
+		$! = Errno::ENOENT() if exists &Errno::ENOENT;
+		return undef;
+	}
+
 	sysopen my $fh, $file, $emode, $perms or return undef;
 	$fh
 }
